@@ -2,6 +2,7 @@
 //  repair tool). See http://parchive.sourceforge.net for details of PAR 2.0.
 //
 //  Copyright (c) 2003 Peter Brian Clements
+//  Copyright (c) 2019 Michael D. Nahas
 //
 //  par2cmdline is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-#include "par2cmdline.h"
+#include "libpar2internal.h"
 
 #ifdef _MSC_VER
 #ifdef _DEBUG
@@ -33,15 +34,15 @@ static char THIS_FILE[]=__FILE__;
 #endif
 
 
-#ifdef WIN32
+#ifdef _WIN32
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 #define OffsetType __int64
 #define MaxOffset 0x7fffffffffffffffI64
-#define LengthType unsigned int
-#define MaxLength 0xffffffffUL
 
-DiskFile::DiskFile(void)
+DiskFile::DiskFile(std::ostream &sout, std::ostream &serr)
+: sout(&sout)
+, serr(&serr)
 {
   filename = "";
   filesize = 0;
@@ -52,6 +53,7 @@ DiskFile::DiskFile(void)
   exists = false;
 }
 
+
 DiskFile::~DiskFile(void)
 {
   if (hFile != INVALID_HANDLE_VALUE)
@@ -78,7 +80,7 @@ bool DiskFile::CreateParentDirectory(string _pathname)
     {
       DWORD error = ::GetLastError();
 
-      cerr << "Could not create the " << path << " directory: " << ErrorMessage(error) << endl;
+      *serr << "Could not create the " << path << " directory: " << ErrorMessage(error) << endl;
 
       return false;
     }
@@ -104,7 +106,7 @@ bool DiskFile::Create(string _filename, u64 _filesize)
   {
     DWORD error = ::GetLastError();
 
-    cerr << "Could not create \"" << _filename << "\": " << ErrorMessage(error) << endl;
+    *serr << "Could not create \"" << _filename << "\": " << ErrorMessage(error) << endl;
 
     return false;
   }
@@ -120,7 +122,7 @@ bool DiskFile::Create(string _filename, u64 _filesize)
     {
       DWORD error = ::GetLastError();
 
-      cerr << "Could not set size of \"" << _filename << "\": " << ErrorMessage(error) << endl;
+      *serr << "Could not set size of \"" << _filename << "\": " << ErrorMessage(error) << endl;
 
       ::CloseHandle(hFile);
       hFile = INVALID_HANDLE_VALUE;
@@ -134,7 +136,7 @@ bool DiskFile::Create(string _filename, u64 _filesize)
     {
       DWORD error = ::GetLastError();
 
-      cerr << "Could not set size of \"" << _filename << "\": " << ErrorMessage(error) << endl;
+      *serr << "Could not set size of \"" << _filename << "\": " << ErrorMessage(error) << endl;
 
       ::CloseHandle(hFile);
       hFile = INVALID_HANDLE_VALUE;
@@ -152,7 +154,7 @@ bool DiskFile::Create(string _filename, u64 _filesize)
 
 // Write some data to disk
 
-bool DiskFile::Write(u64 _offset, const void *buffer, size_t length)
+bool DiskFile::Write(u64 _offset, const void *buffer, size_t length, LengthType maxlength)
 {
   assert(hFile != INVALID_HANDLE_VALUE);
 
@@ -167,38 +169,46 @@ bool DiskFile::Write(u64 _offset, const void *buffer, size_t length)
     {
       DWORD error = ::GetLastError();
 
-      cerr << "Could not write " << (u64)length << " bytes to \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
+      *serr << "Could not write " << (u64)length << " bytes to \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
 
       return false;
     }
     offset = _offset;
   }
 
-  if (length > MaxLength)
-  {
-    cerr << "Could not write " << (u64)length << " bytes to \"" << filename << "\" at offset " << _offset << ": " << "Write too long" << endl;
 
-    return false;
-  }
+  while (length > 0) {
 
-  DWORD write = (LengthType)length;
-  DWORD wrote;
+    DWORD write;
+    if (length > maxlength)
+      write = maxlength;
+    else
+      write = (LengthType) length;
+    DWORD wrote = 0;
 
-  // Write the data
-  if (!::WriteFile(hFile, buffer, write, &wrote, NULL))
-  {
-    DWORD error = ::GetLastError();
+    // Write the data
+    if (!::WriteFile(hFile, buffer, write, &wrote, NULL))
+    {
+      DWORD error = ::GetLastError();
+      
+      *serr << "Could not write " << write << " bytes to \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
+      
+      return false;
+    }
 
-    cerr << "Could not write " << (u64)length << " bytes to \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
+    if (wrote != write)
+    {
+      *serr << "INFO: Incomplete write to \"" << filename << "\" at offset " << _offset << ".  Expected to write " << write << " bytes and wrote " << wrote << " bytes." << endl;
+    }
 
-    return false;
-  }
-
-  offset += length;
-
-  if (filesize < offset)
-  {
-    filesize = offset;
+    offset += wrote;
+    length -= wrote;
+    buffer = ((char *) buffer) + wrote; 
+    
+    if (filesize < offset)
+    {
+      filesize = offset;
+    }
   }
 
   return true;
@@ -206,7 +216,7 @@ bool DiskFile::Write(u64 _offset, const void *buffer, size_t length)
 
 // Open the file
 
-bool DiskFile::Open(string _filename, u64 _filesize)
+bool DiskFile::Open(const string &_filename, u64 _filesize)
 {
   assert(hFile == INVALID_HANDLE_VALUE);
 
@@ -224,7 +234,7 @@ bool DiskFile::Open(string _filename, u64 _filesize)
     case ERROR_PATH_NOT_FOUND:
       break;
     default:
-      cerr << "Could not open \"" << _filename << "\": " << ErrorMessage(error) << endl;
+      *serr << "Could not open \"" << _filename << "\": " << ErrorMessage(error) << endl;
     }
 
     return false;
@@ -238,7 +248,7 @@ bool DiskFile::Open(string _filename, u64 _filesize)
 
 // Read some data from disk
 
-bool DiskFile::Read(u64 _offset, void *buffer, size_t length)
+bool DiskFile::Read(u64 _offset, void *buffer, size_t length, LengthType maxlength)
 {
   assert(hFile != INVALID_HANDLE_VALUE);
 
@@ -253,35 +263,44 @@ bool DiskFile::Read(u64 _offset, void *buffer, size_t length)
     {
       DWORD error = ::GetLastError();
 
-      cerr << "Could not read " << (u64)length << " bytes from \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
+      *serr << "Could not read " << (u64)length << " bytes from \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
 
       return false;
     }
     offset = _offset;
   }
 
-  if (length > MaxLength)
-  {
-    cerr << "Could not read " << (u64)length << " bytes from \"" << filename << "\" at offset " << _offset << ": " << "Read too long" << endl;
+  while (length > 0) {
+  
+    DWORD want;
+    if (length > maxlength)
+      want = maxlength;
+    else
+      want = (LengthType)length;
+    DWORD got = 0;
 
-    return false;
+    // Read the data
+    if (!::ReadFile(hFile, buffer, want, &got, NULL))
+    {
+      DWORD error = ::GetLastError();
+
+      *serr << "Could not read " << (u64)length << " bytes from \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
+
+      return false;
+    }
+
+    if (want != got)
+    {
+      *serr << "Incomplete read from \"" << filename << "\" at offset " << offset << ".  Tried to read " << want << " bytes and received " << got << " bytes." << endl;
+    }
+    
+    offset += got;
+    length -= got;
+    buffer = ((char *) buffer) + got; 
+
+    // write updates filesize.  Do we want to do that here?
   }
 
-  DWORD want = (LengthType)length;
-  DWORD got;
-
-  // Read the data
-  if (!::ReadFile(hFile, buffer, want, &got, NULL))
-  {
-    DWORD error = ::GetLastError();
-
-    cerr << "Could not read " << (u64)length << " bytes from \"" << filename << "\" at offset " << _offset << ": " << ErrorMessage(error) << endl;
-
-    return false;
-  }
-
-  offset += length;
-
   return true;
 }
 
@@ -357,7 +376,7 @@ string DiskFile::GetCanonicalPathname(string filename)
   return longname;
 }
 
-list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
+std::unique_ptr< list<string> > DiskFile::FindFiles(string path, string wildcard, bool recursive)
 {
   // check path, if not ending with path separator, add one
   char pathend = *path.rbegin();
@@ -384,9 +403,10 @@ list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
           continue;
         }
 
-        list<string> *dirmatches;
         string nwwildcard="*";
-        dirmatches = DiskFile::FindFiles(fd.cFileName, nwwildcard, true);
+	std::unique_ptr< list<string> > dirmatches(
+						 DiskFile::FindFiles(fd.cFileName, nwwildcard, true)
+						 );
 
         matches->merge(*dirmatches);
       }
@@ -394,7 +414,7 @@ list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
     ::FindClose(h);
   }
 
-  return matches;
+  return std::unique_ptr< list<string> >(matches);
 }
 
 u64 DiskFile::GetFileSize(string filename)
@@ -410,8 +430,14 @@ u64 DiskFile::GetFileSize(string filename)
   }
 }
 
+bool DiskFile::FileExists(string filename)
+{
+  struct _stati64 st;
+  return ((0 == _stati64(filename.c_str(), &st)) && (0 != (st.st_mode & S_IFREG)));
+}
+
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-#else // !WIN32
+#else // !_WIN32
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_FSEEKO
@@ -428,10 +454,10 @@ u64 DiskFile::GetFileSize(string filename)
 # endif
 #endif
 
-#define LengthType unsigned int
-#define MaxLength 0xffffffffUL
 
-DiskFile::DiskFile(void)
+DiskFile::DiskFile(std::ostream &sout, std::ostream &serr)
+: sout(&sout)
+, serr(&serr)
 {
   //filename;
   filesize = 0;
@@ -442,6 +468,7 @@ DiskFile::DiskFile(void)
   exists = false;
 }
 
+
 DiskFile::~DiskFile(void)
 {
   if (file != 0)
@@ -466,7 +493,7 @@ bool DiskFile::CreateParentDirectory(string _pathname)
 
     if (mkdir(path.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH))
     {
-      cerr << "Could not create the " << path << " directory: " << strerror(errno) << endl;
+      *serr << "Could not create the " << path << " directory: " << strerror(errno) << endl;
       return false;
     }
   }
@@ -485,17 +512,25 @@ bool DiskFile::Create(string _filename, u64 _filesize)
   if (!DiskFile::CreateParentDirectory(filename))
     return false;
 
+  // This is after CreateParentDirectory because
+  // the Windows code would error out after too.
+  if (FileExists(filename))
+  {
+    *serr << "Could not create \"" << _filename << "\": File already exists." << endl;
+    return false;
+  }
+
   file = fopen(_filename.c_str(), "wb");
   if (file == 0)
   {
-    cerr << "Could not create " << _filename << ": " << strerror(errno) << endl;
+    *serr << "Could not create " << _filename << ": " << strerror(errno) << endl;
 
     return false;
   }
 
   if (_filesize > (u64)MaxOffset)
   {
-    cerr << "Requested file size for " << _filename << " is too large." << endl;
+    *serr << "Requested file size for " << _filename << " is too large." << endl;
     return false;
   }
 
@@ -503,7 +538,7 @@ bool DiskFile::Create(string _filename, u64 _filesize)
   {
     if (fseek(file, (OffsetType)_filesize-1, SEEK_SET))
     {
-      cerr << "Could not set end of file of " << _filename << ": " << strerror(errno) << endl;
+      *serr << "Could not set end of file of " << _filename << ": " << strerror(errno) << endl;
 
       fclose(file);
       file = 0;
@@ -513,7 +548,7 @@ bool DiskFile::Create(string _filename, u64 _filesize)
 
     if (1 != fwrite(&_filesize, 1, 1, file))
     {
-      cerr << "Could not set end of file of " << _filename << ": " << strerror(errno) << endl;
+      *serr << "Could not set end of file of " << _filename << ": " << strerror(errno) << endl;
 
       fclose(file);
       file = 0;
@@ -530,7 +565,7 @@ bool DiskFile::Create(string _filename, u64 _filesize)
 
 // Write some data to disk
 
-bool DiskFile::Write(u64 _offset, const void *buffer, size_t length)
+bool DiskFile::Write(u64 _offset, const void *buffer, size_t length, LengthType maxlength)
 {
   assert(file != 0);
 
@@ -538,36 +573,42 @@ bool DiskFile::Write(u64 _offset, const void *buffer, size_t length)
   {
     if (_offset > (u64)MaxOffset)
     {
-        cerr << "Could not write " << (u64)length << " bytes to " << filename << " at offset " << _offset << endl;
+        *serr << "Could not write " << (u64)length << " bytes to " << filename << " at offset " << _offset << endl;
       return false;
     }
 
 
     if (fseek(file, (OffsetType)_offset, SEEK_SET))
     {
-      cerr << "Could not write " << (u64)length << " bytes to " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
+      *serr << "Could not write " << (u64)length << " bytes to " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
       return false;
     }
     offset = _offset;
   }
 
-  if (length > MaxLength)
-  {
-    cerr << "Could not write " << (u64)length << " bytes to " << filename << " at offset " << _offset << endl;
-    return false;
-  }
+  while (length > 0) {
 
-  if (1 != fwrite(buffer, (LengthType)length, 1, file))
-  {
-    cerr << "Could not write " << (u64)length << " bytes to " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
-    return false;
-  }
+    LengthType write;
+    if (length > maxlength)
+      write = maxlength;
+    else
+      write = length;
+    
+    LengthType wrote = fwrite(buffer, 1, write, file);
+    if (wrote != write)
+    {
+      *serr << "Could not write " << (u64)length << " bytes to " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
+      return false;
+    }
 
-  offset += length;
+    offset += wrote;
+    length -= wrote;
+    buffer = ((char *) buffer) + wrote; 
 
-  if (filesize < offset)
-  {
-    filesize = offset;
+    if (filesize < offset)
+    {
+      filesize = offset;
+    }
   }
 
   return true;
@@ -575,7 +616,7 @@ bool DiskFile::Write(u64 _offset, const void *buffer, size_t length)
 
 // Open the file
 
-bool DiskFile::Open(string _filename, u64 _filesize)
+bool DiskFile::Open(const string &_filename, u64 _filesize)
 {
   assert(file == 0);
 
@@ -584,7 +625,7 @@ bool DiskFile::Open(string _filename, u64 _filesize)
 
   if (_filesize > (u64)MaxOffset)
   {
-    cerr << "File size for " << _filename << " is too large." << endl;
+    *serr << "File size for " << _filename << " is too large." << endl;
     return false;
   }
 
@@ -602,7 +643,7 @@ bool DiskFile::Open(string _filename, u64 _filesize)
 
 // Read some data from disk
 
-bool DiskFile::Read(u64 _offset, void *buffer, size_t length)
+bool DiskFile::Read(u64 _offset, void *buffer, size_t length, LengthType maxlength)
 {
   assert(file != 0);
 
@@ -610,33 +651,44 @@ bool DiskFile::Read(u64 _offset, void *buffer, size_t length)
   {
     if (_offset > (u64)MaxOffset)
     {
-      cerr << "Could not read " << (u64)length << " bytes from " << filename << " at offset " << _offset << endl;
+      *serr << "Could not read " << (u64)length << " bytes from " << filename << " at offset " << _offset << endl;
       return false;
     }
 
 
     if (fseek(file, (OffsetType)_offset, SEEK_SET))
     {
-      cerr << "Could not read " << (u64)length << " bytes from " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
+      *serr << "Could not read " << (u64)length << " bytes from " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
       return false;
     }
     offset = _offset;
   }
 
-  if (length > MaxLength)
-  {
-    cerr << "Could not read " << (u64)length << " bytes from " << filename << " at offset " << _offset << endl;
-    return false;
+
+  while (length > 0) {
+
+    LengthType want;
+    if (length > maxlength)
+      want = maxlength;
+    else
+      want = length;
+
+    LengthType got = fread(buffer, 1, want, file);
+    if (got != want) 
+    {
+      // NOTE: This can happen on error or when hitting the end-of-file.
+      
+      *serr << "Could not read " << (u64)length << " bytes from " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
+      return false;
+    }
+
+    offset += got;
+    length -= got;
+    buffer = ((char *) buffer) + got;
+
+    // Write() updates filesize.  Should we do that here too?
   }
-
-  if (1 != fread(buffer, (LengthType)length, 1, file))
-  {
-    cerr << "Could not read " << (u64)length << " bytes from " << filename << " at offset " << _offset << ": " << strerror(errno) << endl;
-    return false;
-  }
-
-  offset += length;
-
+  
   return true;
 }
 
@@ -722,7 +774,7 @@ string DiskFile::GetCanonicalPathname(string filename)
   return result;
 }
 
-list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
+std::unique_ptr< list<string> > DiskFile::FindFiles(string path, string wildcard, bool recursive)
 {
   // check path, if not ending with path separator, add one
   char pathend = *path.rbegin();
@@ -766,10 +818,10 @@ list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
                   recursive == true)
               {
 
-                list<string> *dirmatches;
                 string nwwildcard="*";
-                dirmatches = DiskFile::FindFiles(fn, nwwildcard, true);
-
+                std::unique_ptr< list<string> > dirmatches(
+							 DiskFile::FindFiles(fn, nwwildcard, true)
+							 );
                 matches->merge(*dirmatches);
               }
               else if (S_ISREG(st.st_mode))
@@ -803,9 +855,10 @@ list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
                     recursive == true)
                 {
 
-                  list<string> *dirmatches;
                   string nwwildcard="*";
-                  dirmatches = DiskFile::FindFiles(fn, nwwildcard, true);
+		  std::unique_ptr< list<string> > dirmatches(
+							   DiskFile::FindFiles(fn, nwwildcard, true)
+							   );
 
                   matches->merge(*dirmatches);
                 }
@@ -831,10 +884,10 @@ list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
       if (S_ISDIR(st.st_mode) &&
           recursive == true)
       {
-
-        list<string> *dirmatches;
         string nwwildcard="*";
-        dirmatches = DiskFile::FindFiles(fn, nwwildcard, true);
+	std::unique_ptr< list<string> > dirmatches(
+						 DiskFile::FindFiles(fn, nwwildcard, true)
+						 );
 
         matches->merge(*dirmatches);
       }
@@ -845,7 +898,7 @@ list<string>* DiskFile::FindFiles(string path, string wildcard, bool recursive)
     }
   }
 
-  return matches;
+  return std::unique_ptr< list<string> >(matches);
 }
 
 u64 DiskFile::GetFileSize(string filename)
@@ -860,6 +913,12 @@ u64 DiskFile::GetFileSize(string filename)
     return 0;
   }
 }
+
+bool DiskFile::FileExists(string filename)
+{
+  struct stat st;
+  return ((0 == stat(filename.c_str(), &st)) && (0 != (st.st_mode & S_IFREG)));
+}
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 #endif
 
@@ -870,7 +929,7 @@ bool DiskFile::Open(void)
   return Open(_filename);
 }
 
-bool DiskFile::Open(string _filename)
+bool DiskFile::Open(const string &_filename)
 {
   return Open(_filename, GetFileSize(_filename));
 }
@@ -879,7 +938,7 @@ bool DiskFile::Open(string _filename)
 
 bool DiskFile::Delete(void)
 {
-#ifdef WIN32
+#ifdef _WIN32
   assert(hFile == INVALID_HANDLE_VALUE);
 #else
   assert(file == 0);
@@ -887,11 +946,12 @@ bool DiskFile::Delete(void)
 
   if (filename.size() > 0 && 0 == unlink(filename.c_str()))
   {
+    exists = false;
     return true;
   }
   else
   {
-    cerr << "Cannot delete " << filename << endl;
+    *serr << "Cannot delete " << filename << endl;
 
     return false;
   }
@@ -935,79 +995,6 @@ void DiskFile::SplitRelativeFilename(string filename, string basepath, string &n
   name.erase(0, basepath.length());
 }
 
-bool DiskFile::FileExists(string filename)
-{
-  struct stat st;
-  return ((0 == stat(filename.c_str(), &st)) && (0 != (st.st_mode & S_IFREG)));
-}
-
-// Take a filename from a PAR2 file and replace any characters
-// which would be illegal for a file on disk
-string DiskFile::TranslateFilename(string filename)
-{
-  string result;
-
-  string::iterator p = filename.begin();
-  while (p != filename.end())
-  {
-    unsigned char ch = *p;
-
-    bool ok = true;
-#ifdef WIN32
-    if (ch < 32)
-    {
-      ok = false;
-    }
-    else
-    {
-      switch (ch)
-      {
-      case '"':
-      case '*':
-      case ':':
-      case '<':
-      case '>':
-      case '?':
-      case '|':
-        ok = false;
-      }
-    }
-#else
-    if (ch < 32)
-    {
-      ok = false;
-    }
-#endif
-
-    // replace unix / to windows \ or windows \ to unix /
-#ifdef WIN32
-    if (ch == '/') {
-      ch = '\\';
-    }
-#else
-    if (ch == '\\') {
-      ch = '/';
-    }
-#endif
-
-
-    if (ok)
-    {
-      result += ch;
-    }
-    else
-    {
-      // convert problem characters to hex
-      result += ((ch >> 4) < 10) ? (ch >> 4) + '0' : (ch >> 4) + 'A'-10;
-      result += ((ch & 0xf) < 10) ? (ch & 0xf) + '0' : (ch & 0xf) + 'A'-10;
-    }
-
-    ++p;
-  }
-
-  return result;
-}
-
 bool DiskFile::Rename(void)
 {
   char newname[_MAX_PATH+1];
@@ -1017,15 +1004,15 @@ bool DiskFile::Rename(void)
 
   do
   {
-    int length = snprintf(newname, _MAX_PATH, "%s.%d", filename.c_str(), ++index);
+    int length = snprintf(newname, _MAX_PATH, "%s.%u", filename.c_str(), (unsigned int) ++index);
     if (length < 0)
     {
-      cerr << filename << " cannot be renamed." << endl;
+      *serr << filename << " cannot be renamed." << endl;
       return false;
     }
     else if (length > _MAX_PATH)
     {
-      cerr << filename << " pathlength is more than " << _MAX_PATH << "." << endl;
+      *serr << filename << " pathlength is more than " << _MAX_PATH << "." << endl;
       return false;
     }
     newname[length] = 0;
@@ -1036,7 +1023,7 @@ bool DiskFile::Rename(void)
 
 bool DiskFile::Rename(string _filename)
 {
-#ifdef WIN32
+#ifdef _WIN32
   assert(hFile == INVALID_HANDLE_VALUE);
 #else
   assert(file == 0);
@@ -1050,13 +1037,13 @@ bool DiskFile::Rename(string _filename)
   }
   else
   {
-    cerr << filename << " cannot be renamed to " << _filename << endl;
+    *serr << filename << " cannot be renamed to " << _filename << endl;
 
     return false;
   }
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 string DiskFile::ErrorMessage(DWORD error)
 {
   string result;
@@ -1125,3 +1112,24 @@ DiskFile* DiskFileMap::Find(string filename) const
 
   return (f != diskfilemap.end()) ?  f->second : 0;
 }
+
+
+FileSizeCache::FileSizeCache()
+{
+}
+
+u64 FileSizeCache::get(const string &filename) {
+  map<string, u64>::const_iterator f = cache.find(filename);
+  if (f != cache.end())
+    return f->second;
+
+  // go to disk
+  u64 filesize = DiskFile::GetFileSize(filename);
+
+  cache.insert(pair<string,u64>(filename, filesize));
+  //  pair<map<string,u64>::const_iterator,bool> location = cache.insert(pair<string,u64>(filename, filesize));
+  //  if (!location.second) {
+  //    throw exception?
+  //  }
+  return filesize;
+}
